;******************   Motorola DSP5630X   ******************
;***                                                     ***
;***            Exercise 2 : Addressing Modes            ***
;***                                                     ***
;***                                                     ***
;***********************************************************

table	equ 10
inc	equ 3
points  equ $8
revdata equ $200

	SECTION x_modulo_data
	GLOBAL const
	org x:	
const	dc 0,1,2,3,4,5,6,7,8,9	; data for Modulo Table
	ENDSEC

	SECTION x_revbit_data
	GLOBAL orgdatar
	org x:	
orgdatar			; Begining of original data
	dc 0.1,0.2,0.3,0.4	; e.g. Real Part of FFT
	dc 0.5,0.6,0.7,0.8
	ENDSEC

	SECTION y_revbit_data
	GLOBAL orgdatai
	org y:
orgdatai			; Begining of original data
	dc 0.15,0.25,0.35,0.45	; e.g. Imaginary Part of FFT
	dc 0.55,0.65,0.75,0.85
	ENDSEC

	SECTION ex2_code
	org p:

ex2_start

; REGISTER DIRECT MODE:-
; Transfer the contents of the 24-bit X0 data reg.
; to the 24-bit A1 accumulator.

	move #$876543,x0
	move x0,a1


; ADDRESS REGISTER INDIRECT MODE:-
; No Update (Rn)
; Transfer the contents of Accumulator Register A1 to the 
; X-Memory location pointed to by Address Register R0

indirect_start
	move #$1000,r0
	move #$123456,a1
	move a1,x:(r0)

; Postincrement by one (Rn)+
; Transfer the contents of Accumulator Register B0 to the 
; Y-Memory location pointed to by Address Register R1. 
; Once the transfer is complete, R1 is incremented by one

postinc_start
	move #$2000,r1
	move #$654321,b0
	move b0,y:(r1)+


; Predecrement by one -(Rn)
; Predecrement Address Register R3 by one and trasnsfer 
; the X-Memory location pointed to by the decremented 
; Address Register R3 to Accumulator Register B

predec_start
	move #$3FFF,r2
	move #$4000,r3 
	move #$121212,x0
	move x0,x:(r2)
	move x:-(r3),b

; Postincrement by Offset (Rn)+Nn
; Transfer the contents of Data Input Register X1 to the 
; X-Memory location pointed to by Address Register R4. 
; Once the transfer is complete, R4 is updated by adding 
; the offset contained in Offset Register N4 to the 
; contents of R4

incoffset_start
	move #$987654,x1
	move #$5000,r4
	move #$4,n4
	move x1,x:(r4)+n4

; Indexed by Offset (Rn+Nn)
; Transfer the contents of Data Input Register Y0 to the 
; X-Memory location pointed to by the summation of the 
; contents of the Address Register R6 and the Offset 
; Register N6. Note that Address Register R6 is not 
; updated

indexoff_start
	move #$ABCDEF,y0
	move #$7000,r6
	move #$5,n6
	move y0,x:(r6+n6)

; Modulo Addressing 

modulo_start

	move #const,r0
	move #table-1,m0
	move #inc,n0
	do #table,_end_modulo_addr
	move x:(r0)+n0,x0

_end_modulo_addr

; Bit Reverse Addressing  

bitrev_start

	move #orgdatar,r0	; Point to Orginal Input data
	move #orgdatai,r4	; address sequence
	move #0,m0		; Set up for Bit-Revesred 
        move m0,m4              ; addressing
	move #points/2,n0       ;               k-1    3-1
	move n0,n4       	; Offset Reg = 2   -> 2  = 4

	move #revdata,r1	; Point to Reversed address
        move r1,r5              ; sequence Output data
	move #-1,m1		; Set up for Linear 
        move m1,m5              ; addressing

	do #points,_end_bit_reverse
	move x:(r0)+n0,a y:(r4)+n4,b
	move a,x:(r1)+ b,y:(r5)+

_end_bit_reverse


; IMMEDIATE  DATA:-
; Transfer the immediate 24-bit data value $123456 to 
; Accumulator Register A0

immed_start
	move #$123456,a0

; Positive Immediate into 56-bit Accumulator
; Transfer the immediate value $654321 to Accumulator 
; Register A. Note that the Accumulator Register A1 is 
; loaded and that Accumulator Register A2 is sign- 
; exteneded from A1 i.e. A2 holds the value $00 indicating 
; that the value stored in A1 is positive

posimmed_start
	move #$654321,a

; Negative Immediate into 56-bit Accumulator
; Transfer the immediate value $876543 to Accumulator 
; Register B. Note that the Accumulator Register B1 is 
; loaded and that Accumulator Register B2 is sign- 
; exteneded from B1 i.e. B2 holds the value $FF indicating 
; that the value stored in B1 is negative

negimmed_start
	move #$876543,b


; IMMEDIATE  SHORT:-
; IMMEDIATE SHORT DATA INTO 24-BIT REGISTERS
; Transfer the immediate short data value $FE to 
; Accumulator Register A1. Note that the immediate data 
; $FE is interpreted as an unsigned interger and is 
; transfered into the least significant bits of A1

intshort_start
	move #$FE,a1

; Transfer the immediate short data value $FE to Data Input 
; Register Y1. Note that the immediate data $FE is 
; interpreted as a signed fraction and is transfered into 
; the most significant bits of Y1

fractshort_start
	move #$FE,y1

; IMMEDIATE SHORT DATA INTO 56-BIT ACCUMULATORS
; Transfer the immediate short data value $87 to the 
; Accumulator Register B. Note that the immediate data $87 
; is interpreted as a signed fraction and is transfered into
; the most significant bits of the Accumulator Register B 
; and that Accumulator Register B2 is sign-exteneded

negfractshort_start
	move #$87,b


; ABSOLUTE  ADDRESSING
; Transfer the contents of the Y-Memory location pointed to 
; by the instruction extension word to Accumulator Register 
; B0

absolute_start
	move #$765432,y1
	move y1,y:$001000
	move y:$1000,b0


;SHORT  ADDRESSING
; Transfer the contents of the Accumulator Register A1
; to the X-Memory location short address $21

abshort_start
	move #$765432,a1
	move a1,x:$21


; PARALLEL DATA MOVES
; X and Y Memory Parallel Data Move

parallel_start
	add y,b b,x:(r3)+n3 y:(r7)+n7,y1

	ENDSEC
