;****************************************************************
; FILENAME : ex4_fir.asm
;
; ROUTINES : fir_filter
;****************************************************************
	TITLE	'EX4 FIR'
	section fir_filter
	xdef	fir_filter
	org	p:
;*****************************************************************
; SUBROUTINE	: fir_filter
;
; PURPOSE	: Implements the fir_filter loop
;
; INPUTS	r4 points to list of taps, stored in Y data memory at
;		a location which is suitable for use as a modulo 
;		buffer. 
;		
;		r0 points to the input to the filter which is a list 
;		of input values stored in X memory
;
; 		r2 points to the space in y memory where the ouput values
;		are to be stored
;
;		r3 holds the number of taps in the filter
;
;		r1 holds the number of inputs which are to be processed 
;		this call
;
; OUTPUTS	output values in Y memory which was pointer to by r2     
;
;*******************************************************************
fir_filter
	move	x:(r3)-,x0		;dummy read to decrement r3
	move	r3,m4			;set up r4 as a modulo
					;NUM_OF_TAPS buffer
	move	r3,n0	
;Main FIR loop
	dor	r1,for_every_input_value
	clr	a	x:(r0)+,x0	y:(r4)+,y0
					;Clear the accumulator 
					;and load the first data 
					;value and filter coefficient
	rep	n0			;Repeat the following 
					;instruction (NUM_OF_TAPS-1)
					;times
	mac	x0,y0,a	x:(r0)+,x0	y:(r4)+,y0
					;Multiply and accumulate,
					;load in next data value 
					;and coefficient
	mac	x0,y0,a	x:(r0)-n0,x0
					;For final tap multiply and
					;accumulate 
					;r0 has been incremented 10
					;times, use n0 to take it
					;back 9 places in the input
					;buffer
	move	a1,y:(r2)+		;store result in output
					;buffer
for_every_input_value
;End of main FIR loop
	move	#$FFFF,m4		;restore modifier register
					;to RESET values

	rts				;end of subroutine
	endsec
