;*************************************************************************
; FILENAME	:	ex6_main.asm
; ROUTINES	:	ex6_main
;*************************************************************************
;************************************************************************
;SUBROUTINE	:	ex6_main.asm
;PURPOSE	
;		MATRIX MULTIPLICATION EXAMPLE 
;		Code and Example Data 
;************************************************************************
;
; In this example two matrices (A and B) are multiplied together to give result C.
;
; M, N and P are the matrix dimensions:
;    Input matrix A is a 2x4 (NxM) matrix
;    Input matrix B is a 4x3 (MxP) matrix
;    Output matrix C is a 2x3 (NxP) matrix
;
; The following fractional data is used for A, B and C:
;             A                          B                       C
; [ 0.10, 0.11, 0.12, 0.13 ] x [ 0.32, 0.52, 0.72 ] = [ 0.162, 0.254, 0.346 ]
; [ 0.20, 0.21, 0.22, 0.23 ]   [ 0.34, 0.54, 0.74 ]   [ 0.302, 0.474, 0.646 ]
;                              [ 0.36, 0.56, 0.76 ]
;                              [ 0.38, 0.58, 0.78 ]
;
	section	matrix_mult
; Define matrix sizes
M	equ	4	; Input matrix A is a 2x4 (NxM) matrix
P	equ	3	; Input matrix B is a 4x3 (MxP) matrix
N	equ	2	; Output matrix C is a 2x3 (NxP) matrix
; Initialise matrix contents
; Input matrix A - locate in x memory
	org	x:
	buffer m,N*M
A_ptr	dc	0.10,0.11,0.12,0.13
	dc	0.20,0.21,0.22,0.23
	endbuf
; Input matrix B - locate in y memory
	org	y:
	buffer m,M*P
B_ptr	dc	0.32,0.52,0.72
	dc	0.34,0.54,0.74
	dc	0.36,0.56,0.76
	dc	0.38,0.58,0.78
	endbuf
; Clear result matrix C - locate in x memory
	org	x:
C_ptr	dc	0,0,0
	dc	0,0,0
; Now the code to multiply matrices A and B
	org	p:
ex6_start
	move	#A_ptr,r0	; Set up input matrix A address pointer
	move	#C_ptr,r1	; Set up result matrix C address pointer
	move	#P,n4		; Set up address offset value for B matrix
	move	#(M*P-1),m4	; Modulo 12 addressing for B matrix
	move	#(M-1),m0	; Modulo 4 addressing for A matrix
	dor	#N,loop1
	move	#B_ptr,r4	; Set up input matrix B address pointer
	dor	#P,loop2
	clr	a	x:(r0)+,x0	y:(r4)+n4,y0	; Clear result accumulator
	rep	#M-1
	mac	x0,y0,a x:(r0)+,x0	y:(r4)+n4,y0
	mac	x0,y0,a
	move	a,x:(r1)+	; Store Result
	lua	(r4+1),r4	; Move to next column in B matrix
loop2				; End of loop2
	lua	(r0+M),r0	; Move to next row in A matrix
loop1				; End of loop1
	bra	loop1	
	endsec			; End of section matrix_mult
; End of file
